import logging

from tqdm import tqdm

from karaokatalog.deduplicate.find_duplicates import find_duplicates
from karaokatalog.deduplicate.prune import prune
from karaokatalog.get_parser import get_parser
from karaokatalog.Library import Library
from karaokatalog.Song import Song
from karaokatalog.util.get_equivalence_classes import get_equivalence_classes

logging.basicConfig(
    format="%(asctime)s [%(levelname)s] %(message)s", level=logging.INFO
)


if __name__ == "__main__":
    args = get_parser(
        "deduplicate", "Deduplicate UltraStar Deluxe song libraries"
    ).parse_args()
    logging.info("Karaokatalog Deduplication started")

    logging.info("Loading library")
    library = Library.from_dir(args.library_path)
    logging.info("Library loaded")

    logging.info("Finding duplicates (songs with identical title and artist)")
    duplicates = find_duplicates(library.songs_in_unique_dirs)
    logging.info(f"{len(duplicates)} duplicate sets found")

    logging.info("Finding exact duplicates (files are 100% identical)")
    # TODO Make this abortable and restartable
    exact_duplicates = tuple(
        equivalence_class
        for songs in tqdm(duplicates.values(), unit=" duplicates")
        for equivalence_class in get_equivalence_classes(songs, Song.has_identic_files)
    )
    logging.info(f"{len(exact_duplicates)} exact duplicate sets found")

    logging.info("Determining songs to prune")
    pruning_instructions = tuple(
        instruction
        for equivalence_class in tqdm(exact_duplicates, unit=" exact duplicates")
        for instruction in prune(equivalence_class)
    )
    logging.info(
        f"{len(pruning_instructions)} exactly duplicated songs will be deleted"
    )

    logging.warning(f"Deleting {len(pruning_instructions)} songs!")
    for instruction in tqdm(pruning_instructions, unit=" songs"):
        instruction()
    logging.info("Deletion done")

    logging.info("Karaokatalog Deduplication done")
