import logging

from tqdm import tqdm

from karaokatalog.get_parser import get_parser
from karaokatalog.Library import Library
from karaokatalog.organize.move import move

logging.basicConfig(
    format="%(asctime)s [%(levelname)s] %(message)s", level=logging.INFO
)


if __name__ == "__main__":
    args = get_parser(
        "organize", "Organize UltraStar Deluxe song libraries"
    ).parse_args()
    logging.info("Karaokatalog Organization started")

    logging.info("Loading library")
    library = Library.from_dir(args.library_path)
    logging.info("Library loaded")

    logging.info("Generating move instructions")
    move_instructions = move(library.songs, args.library_path)
    logging.info(f"{len(move_instructions)} move instructions generated")

    logging.warning(f"Moving {len(move_instructions)} songs!")
    for instruction in tqdm(move_instructions, unit=" songs"):
        instruction()
    logging.info("Moving done")

    logging.info("Karaokatalog Organization done")
